# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, cast
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from .....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ....._base_client import AsyncPaginator, make_request_options
from .....types.workers.beta.workers import version_get_params, version_list_params, version_create_params
from .....types.workers.beta.workers.version import Version
from .....types.workers.beta.workers.version_delete_response import VersionDeleteResponse

__all__ = ["VersionsResource", "AsyncVersionsResource"]


class VersionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> VersionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return VersionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VersionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return VersionsResourceWithStreamingResponse(self)

    def create(
        self,
        worker_id: str,
        *,
        account_id: str,
        deploy: bool | Omit = omit,
        annotations: version_create_params.Annotations | Omit = omit,
        assets: version_create_params.Assets | Omit = omit,
        bindings: Iterable[version_create_params.Binding] | Omit = omit,
        compatibility_date: str | Omit = omit,
        compatibility_flags: SequenceNotStr[str] | Omit = omit,
        containers: Iterable[version_create_params.Container] | Omit = omit,
        limits: version_create_params.Limits | Omit = omit,
        main_module: str | Omit = omit,
        migrations: version_create_params.Migrations | Omit = omit,
        modules: Iterable[version_create_params.Module] | Omit = omit,
        placement: version_create_params.Placement | Omit = omit,
        usage_model: Literal["standard", "bundled", "unbound"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Version:
        """
        Create a new version.

        Args:
          account_id: Identifier.

          worker_id: Identifier for the Worker, which can be ID or name.

          deploy: If true, a deployment will be created that sends 100% of traffic to the new
              version.

          annotations: Metadata about the version.

          assets: Configuration for assets within a Worker.

              [`_headers`](https://developers.cloudflare.com/workers/static-assets/headers/#custom-headers)
              and
              [`_redirects`](https://developers.cloudflare.com/workers/static-assets/redirects/)
              files should be included as modules named `_headers` and `_redirects` with
              content type `text/plain`.

          bindings: List of bindings attached to a Worker. You can find more about bindings on our
              docs:
              https://developers.cloudflare.com/workers/configuration/multipart-upload-metadata/#bindings.

          compatibility_date: Date indicating targeted support in the Workers runtime. Backwards incompatible
              fixes to the runtime following this date will not affect this Worker.

          compatibility_flags: Flags that enable or disable certain features in the Workers runtime. Used to
              enable upcoming features or opt in or out of specific changes not included in a
              `compatibility_date`.

          containers: List of containers attached to a Worker. Containers can only be attached to
              Durable Object classes of this Worker script.

          limits: Resource limits enforced at runtime.

          main_module: The name of the main module in the `modules` array (e.g. the name of the module
              that exports a `fetch` handler).

          migrations: Migrations for Durable Objects associated with the version. Migrations are
              applied when the version is deployed.

          modules: Code, sourcemaps, and other content used at runtime.

              This includes
              [`_headers`](https://developers.cloudflare.com/workers/static-assets/headers/#custom-headers)
              and
              [`_redirects`](https://developers.cloudflare.com/workers/static-assets/redirects/)
              files used to configure
              [Static Assets](https://developers.cloudflare.com/workers/static-assets/).
              `_headers` and `_redirects` files should be included as modules named `_headers`
              and `_redirects` with content type `text/plain`.

          placement: Configuration for
              [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
              Specify mode='smart' for Smart Placement, or one of region/hostname/host.

          usage_model: Usage model for the version.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not worker_id:
            raise ValueError(f"Expected a non-empty value for `worker_id` but received {worker_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/workers/workers/{worker_id}/versions",
                account_id=account_id,
                worker_id=worker_id,
            ),
            body=maybe_transform(
                {
                    "annotations": annotations,
                    "assets": assets,
                    "bindings": bindings,
                    "compatibility_date": compatibility_date,
                    "compatibility_flags": compatibility_flags,
                    "containers": containers,
                    "limits": limits,
                    "main_module": main_module,
                    "migrations": migrations,
                    "modules": modules,
                    "placement": placement,
                    "usage_model": usage_model,
                },
                version_create_params.VersionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"deploy": deploy}, version_create_params.VersionCreateParams),
                post_parser=ResultWrapper[Version]._unwrapper,
            ),
            cast_to=cast(Type[Version], ResultWrapper[Version]),
        )

    def list(
        self,
        worker_id: str,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[Version]:
        """
        List all versions for a Worker.

        Args:
          account_id: Identifier.

          worker_id: Identifier for the Worker, which can be ID or name.

          page: Current page.

          per_page: Items per-page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not worker_id:
            raise ValueError(f"Expected a non-empty value for `worker_id` but received {worker_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/workers/workers/{worker_id}/versions",
                account_id=account_id,
                worker_id=worker_id,
            ),
            page=SyncV4PagePaginationArray[Version],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    version_list_params.VersionListParams,
                ),
            ),
            model=Version,
        )

    def delete(
        self,
        version_id: str,
        *,
        account_id: str,
        worker_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VersionDeleteResponse:
        """
        Delete a version.

        Args:
          account_id: Identifier.

          worker_id: Identifier for the Worker, which can be ID or name.

          version_id: Identifier for the version, which can be a UUID, a UUID prefix (minimum length
              8), or the literal "latest" to operate on the most recently created version.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not worker_id:
            raise ValueError(f"Expected a non-empty value for `worker_id` but received {worker_id!r}")
        if not version_id:
            raise ValueError(f"Expected a non-empty value for `version_id` but received {version_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/workers/workers/{worker_id}/versions/{version_id}",
                account_id=account_id,
                worker_id=worker_id,
                version_id=version_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VersionDeleteResponse,
        )

    def get(
        self,
        version_id: str,
        *,
        account_id: str,
        worker_id: str,
        include: Literal["modules"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Version:
        """
        Get details about a specific version.

        Args:
          account_id: Identifier.

          worker_id: Identifier for the Worker, which can be ID or name.

          version_id: Identifier for the version, which can be a UUID, a UUID prefix (minimum length
              8), or the literal "latest" to operate on the most recently created version.

          include: Whether to include the `modules` property of the version in the response, which
              contains code and sourcemap content and may add several megabytes to the
              response size.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not worker_id:
            raise ValueError(f"Expected a non-empty value for `worker_id` but received {worker_id!r}")
        if not version_id:
            raise ValueError(f"Expected a non-empty value for `version_id` but received {version_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/workers/workers/{worker_id}/versions/{version_id}",
                account_id=account_id,
                worker_id=worker_id,
                version_id=version_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"include": include}, version_get_params.VersionGetParams),
                post_parser=ResultWrapper[Version]._unwrapper,
            ),
            cast_to=cast(Type[Version], ResultWrapper[Version]),
        )


class AsyncVersionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncVersionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVersionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVersionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncVersionsResourceWithStreamingResponse(self)

    async def create(
        self,
        worker_id: str,
        *,
        account_id: str,
        deploy: bool | Omit = omit,
        annotations: version_create_params.Annotations | Omit = omit,
        assets: version_create_params.Assets | Omit = omit,
        bindings: Iterable[version_create_params.Binding] | Omit = omit,
        compatibility_date: str | Omit = omit,
        compatibility_flags: SequenceNotStr[str] | Omit = omit,
        containers: Iterable[version_create_params.Container] | Omit = omit,
        limits: version_create_params.Limits | Omit = omit,
        main_module: str | Omit = omit,
        migrations: version_create_params.Migrations | Omit = omit,
        modules: Iterable[version_create_params.Module] | Omit = omit,
        placement: version_create_params.Placement | Omit = omit,
        usage_model: Literal["standard", "bundled", "unbound"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Version:
        """
        Create a new version.

        Args:
          account_id: Identifier.

          worker_id: Identifier for the Worker, which can be ID or name.

          deploy: If true, a deployment will be created that sends 100% of traffic to the new
              version.

          annotations: Metadata about the version.

          assets: Configuration for assets within a Worker.

              [`_headers`](https://developers.cloudflare.com/workers/static-assets/headers/#custom-headers)
              and
              [`_redirects`](https://developers.cloudflare.com/workers/static-assets/redirects/)
              files should be included as modules named `_headers` and `_redirects` with
              content type `text/plain`.

          bindings: List of bindings attached to a Worker. You can find more about bindings on our
              docs:
              https://developers.cloudflare.com/workers/configuration/multipart-upload-metadata/#bindings.

          compatibility_date: Date indicating targeted support in the Workers runtime. Backwards incompatible
              fixes to the runtime following this date will not affect this Worker.

          compatibility_flags: Flags that enable or disable certain features in the Workers runtime. Used to
              enable upcoming features or opt in or out of specific changes not included in a
              `compatibility_date`.

          containers: List of containers attached to a Worker. Containers can only be attached to
              Durable Object classes of this Worker script.

          limits: Resource limits enforced at runtime.

          main_module: The name of the main module in the `modules` array (e.g. the name of the module
              that exports a `fetch` handler).

          migrations: Migrations for Durable Objects associated with the version. Migrations are
              applied when the version is deployed.

          modules: Code, sourcemaps, and other content used at runtime.

              This includes
              [`_headers`](https://developers.cloudflare.com/workers/static-assets/headers/#custom-headers)
              and
              [`_redirects`](https://developers.cloudflare.com/workers/static-assets/redirects/)
              files used to configure
              [Static Assets](https://developers.cloudflare.com/workers/static-assets/).
              `_headers` and `_redirects` files should be included as modules named `_headers`
              and `_redirects` with content type `text/plain`.

          placement: Configuration for
              [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
              Specify mode='smart' for Smart Placement, or one of region/hostname/host.

          usage_model: Usage model for the version.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not worker_id:
            raise ValueError(f"Expected a non-empty value for `worker_id` but received {worker_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/workers/workers/{worker_id}/versions",
                account_id=account_id,
                worker_id=worker_id,
            ),
            body=await async_maybe_transform(
                {
                    "annotations": annotations,
                    "assets": assets,
                    "bindings": bindings,
                    "compatibility_date": compatibility_date,
                    "compatibility_flags": compatibility_flags,
                    "containers": containers,
                    "limits": limits,
                    "main_module": main_module,
                    "migrations": migrations,
                    "modules": modules,
                    "placement": placement,
                    "usage_model": usage_model,
                },
                version_create_params.VersionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"deploy": deploy}, version_create_params.VersionCreateParams),
                post_parser=ResultWrapper[Version]._unwrapper,
            ),
            cast_to=cast(Type[Version], ResultWrapper[Version]),
        )

    def list(
        self,
        worker_id: str,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Version, AsyncV4PagePaginationArray[Version]]:
        """
        List all versions for a Worker.

        Args:
          account_id: Identifier.

          worker_id: Identifier for the Worker, which can be ID or name.

          page: Current page.

          per_page: Items per-page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not worker_id:
            raise ValueError(f"Expected a non-empty value for `worker_id` but received {worker_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/workers/workers/{worker_id}/versions",
                account_id=account_id,
                worker_id=worker_id,
            ),
            page=AsyncV4PagePaginationArray[Version],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    version_list_params.VersionListParams,
                ),
            ),
            model=Version,
        )

    async def delete(
        self,
        version_id: str,
        *,
        account_id: str,
        worker_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VersionDeleteResponse:
        """
        Delete a version.

        Args:
          account_id: Identifier.

          worker_id: Identifier for the Worker, which can be ID or name.

          version_id: Identifier for the version, which can be a UUID, a UUID prefix (minimum length
              8), or the literal "latest" to operate on the most recently created version.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not worker_id:
            raise ValueError(f"Expected a non-empty value for `worker_id` but received {worker_id!r}")
        if not version_id:
            raise ValueError(f"Expected a non-empty value for `version_id` but received {version_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/workers/workers/{worker_id}/versions/{version_id}",
                account_id=account_id,
                worker_id=worker_id,
                version_id=version_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VersionDeleteResponse,
        )

    async def get(
        self,
        version_id: str,
        *,
        account_id: str,
        worker_id: str,
        include: Literal["modules"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Version:
        """
        Get details about a specific version.

        Args:
          account_id: Identifier.

          worker_id: Identifier for the Worker, which can be ID or name.

          version_id: Identifier for the version, which can be a UUID, a UUID prefix (minimum length
              8), or the literal "latest" to operate on the most recently created version.

          include: Whether to include the `modules` property of the version in the response, which
              contains code and sourcemap content and may add several megabytes to the
              response size.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not worker_id:
            raise ValueError(f"Expected a non-empty value for `worker_id` but received {worker_id!r}")
        if not version_id:
            raise ValueError(f"Expected a non-empty value for `version_id` but received {version_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/workers/workers/{worker_id}/versions/{version_id}",
                account_id=account_id,
                worker_id=worker_id,
                version_id=version_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"include": include}, version_get_params.VersionGetParams),
                post_parser=ResultWrapper[Version]._unwrapper,
            ),
            cast_to=cast(Type[Version], ResultWrapper[Version]),
        )


class VersionsResourceWithRawResponse:
    def __init__(self, versions: VersionsResource) -> None:
        self._versions = versions

        self.create = to_raw_response_wrapper(
            versions.create,
        )
        self.list = to_raw_response_wrapper(
            versions.list,
        )
        self.delete = to_raw_response_wrapper(
            versions.delete,
        )
        self.get = to_raw_response_wrapper(
            versions.get,
        )


class AsyncVersionsResourceWithRawResponse:
    def __init__(self, versions: AsyncVersionsResource) -> None:
        self._versions = versions

        self.create = async_to_raw_response_wrapper(
            versions.create,
        )
        self.list = async_to_raw_response_wrapper(
            versions.list,
        )
        self.delete = async_to_raw_response_wrapper(
            versions.delete,
        )
        self.get = async_to_raw_response_wrapper(
            versions.get,
        )


class VersionsResourceWithStreamingResponse:
    def __init__(self, versions: VersionsResource) -> None:
        self._versions = versions

        self.create = to_streamed_response_wrapper(
            versions.create,
        )
        self.list = to_streamed_response_wrapper(
            versions.list,
        )
        self.delete = to_streamed_response_wrapper(
            versions.delete,
        )
        self.get = to_streamed_response_wrapper(
            versions.get,
        )


class AsyncVersionsResourceWithStreamingResponse:
    def __init__(self, versions: AsyncVersionsResource) -> None:
        self._versions = versions

        self.create = async_to_streamed_response_wrapper(
            versions.create,
        )
        self.list = async_to_streamed_response_wrapper(
            versions.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            versions.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            versions.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["BillingUsageHistoryParams"]


class BillingUsageHistoryParams(TypedDict, total=False):
    account_id: Required[str]

    value_grouping_window: Required[Literal["day", "hour"]]
    """Grouping window for usage data."""

    end_time: Optional[float]
    """End time as Unix timestamp in milliseconds."""

    start_time: Optional[float]
    """Start time as Unix timestamp in milliseconds."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .item import Item as Item
from .quota import Quota as Quota
from .list_item import ListItem as ListItem
from .request_types import RequestTypes as RequestTypes
from .request_constants import RequestConstants as RequestConstants
from .request_list_params import RequestListParams as RequestListParams
from .request_create_params import RequestCreateParams as RequestCreateParams
from .request_update_params import RequestUpdateParams as RequestUpdateParams
from .request_types_response import RequestTypesResponse as RequestTypesResponse
from .request_delete_response import RequestDeleteResponse as RequestDeleteResponse
from .threat_event_edit_params import ThreatEventEditParams as ThreatEventEditParams
from .threat_event_list_params import ThreatEventListParams as ThreatEventListParams
from .threat_event_get_response import ThreatEventGetResponse as ThreatEventGetResponse
from .threat_event_create_params import ThreatEventCreateParams as ThreatEventCreateParams
from .threat_event_edit_response import ThreatEventEditResponse as ThreatEventEditResponse
from .threat_event_list_response import ThreatEventListResponse as ThreatEventListResponse
from .binary_storage_create_params import BinaryStorageCreateParams as BinaryStorageCreateParams
from .threat_event_create_response import ThreatEventCreateResponse as ThreatEventCreateResponse
from .binary_storage_create_response import BinaryStorageCreateResponse as BinaryStorageCreateResponse
from .threat_event_bulk_create_params import ThreatEventBulkCreateParams as ThreatEventBulkCreateParams
from .threat_event_bulk_create_response import ThreatEventBulkCreateResponse as ThreatEventBulkCreateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .database import (
    DatabaseResource,
    AsyncDatabaseResource,
    DatabaseResourceWithRawResponse,
    AsyncDatabaseResourceWithRawResponse,
    DatabaseResourceWithStreamingResponse,
    AsyncDatabaseResourceWithStreamingResponse,
)
from .time_travel import (
    TimeTravelResource,
    AsyncTimeTravelResource,
    TimeTravelResourceWithRawResponse,
    AsyncTimeTravelResourceWithRawResponse,
    TimeTravelResourceWithStreamingResponse,
    AsyncTimeTravelResourceWithStreamingResponse,
)

__all__ = [
    "TimeTravelResource",
    "AsyncTimeTravelResource",
    "TimeTravelResourceWithRawResponse",
    "AsyncTimeTravelResourceWithRawResponse",
    "TimeTravelResourceWithStreamingResponse",
    "AsyncTimeTravelResourceWithStreamingResponse",
    "DatabaseResource",
    "AsyncDatabaseResource",
    "DatabaseResourceWithRawResponse",
    "AsyncDatabaseResourceWithRawResponse",
    "DatabaseResourceWithStreamingResponse",
    "AsyncDatabaseResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["CatchAllMatcherParam"]


class CatchAllMatcherParam(TypedDict, total=False):
    """Matcher for catch-all routing rule."""

    type: Required[Literal["all"]]
    """Type of matcher. Default is 'all'."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["Bookmark"]


class Bookmark(BaseModel):
    id: Optional[str] = None
    """The unique identifier for the Bookmark application."""

    app_launcher_visible: Optional[bool] = None
    """Displays the application in the App Launcher."""

    domain: Optional[str] = None
    """The domain of the Bookmark application."""

    logo_url: Optional[str] = None
    """The image URL for the logo shown in the App Launcher dashboard."""

    name: Optional[str] = None
    """The name of the Bookmark application."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncCursorPagination, AsyncCursorPagination
from ...._base_client import AsyncPaginator, make_request_options
from ....types.security_center.insights import audit_log_list_params, audit_log_list_by_insight_params
from ....types.security_center.insights.audit_log_list_response import AuditLogListResponse
from ....types.security_center.insights.audit_log_list_by_insight_response import AuditLogListByInsightResponse

__all__ = ["AuditLogsResource", "AsyncAuditLogsResource"]


class AuditLogsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AuditLogsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AuditLogsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AuditLogsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AuditLogsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        before: Union[str, datetime] | Omit = omit,
        changed_by: str | Omit = omit,
        cursor: str | Omit = omit,
        field_changed: Literal["status", "user_classification"] | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        per_page: int | Omit = omit,
        since: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPagination[AuditLogListResponse]:
        """
        Lists audit log entries for all Security Center insights in the account or zone,
        showing changes to insight status and classification.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          before: Filter entries changed before this timestamp (RFC 3339).

          changed_by: Filter by the actor that made the change.

          cursor: Opaque cursor for pagination. Use the cursor value from result_info of the
              previous response.

          field_changed: Filter by the field that was changed.

          order: Sort order for results. Use 'asc' for oldest first or 'desc' for newest first.

          per_page: Number of results per page.

          since: Filter entries changed at or after this timestamp (RFC 3339).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/security-center/insights/audit-log",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=SyncCursorPagination[AuditLogListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "before": before,
                        "changed_by": changed_by,
                        "cursor": cursor,
                        "field_changed": field_changed,
                        "order": order,
                        "per_page": per_page,
                        "since": since,
                    },
                    audit_log_list_params.AuditLogListParams,
                ),
            ),
            model=AuditLogListResponse,
        )

    def list_by_insight(
        self,
        issue_id: str,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        before: Union[str, datetime] | Omit = omit,
        changed_by: str | Omit = omit,
        cursor: str | Omit = omit,
        field_changed: Literal["status", "user_classification"] | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        per_page: int | Omit = omit,
        since: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPagination[AuditLogListByInsightResponse]:
        """
        Lists audit log entries for a specific Security Center insight, showing changes
        to its status and classification over time.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          before: Filter entries changed before this timestamp (RFC 3339).

          changed_by: Filter by the actor that made the change.

          cursor: Opaque cursor for pagination. Use the cursor value from result_info of the
              previous response.

          field_changed: Filter by the field that was changed.

          order: Sort order for results. Use 'asc' for oldest first or 'desc' for newest first.

          per_page: Number of results per page.

          since: Filter entries changed at or after this timestamp (RFC 3339).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not issue_id:
            raise ValueError(f"Expected a non-empty value for `issue_id` but received {issue_id!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/security-center/insights/{issue_id}/audit-log",
                issue_id=issue_id,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=SyncCursorPagination[AuditLogListByInsightResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "before": before,
                        "changed_by": changed_by,
                        "cursor": cursor,
                        "field_changed": field_changed,
                        "order": order,
                        "per_page": per_page,
                        "since": since,
                    },
                    audit_log_list_by_insight_params.AuditLogListByInsightParams,
                ),
            ),
            model=AuditLogListByInsightResponse,
        )


class AsyncAuditLogsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAuditLogsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAuditLogsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAuditLogsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAuditLogsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        before: Union[str, datetime] | Omit = omit,
        changed_by: str | Omit = omit,
        cursor: str | Omit = omit,
        field_changed: Literal["status", "user_classification"] | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        per_page: int | Omit = omit,
        since: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AuditLogListResponse, AsyncCursorPagination[AuditLogListResponse]]:
        """
        Lists audit log entries for all Security Center insights in the account or zone,
        showing changes to insight status and classification.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          before: Filter entries changed before this timestamp (RFC 3339).

          changed_by: Filter by the actor that made the change.

          cursor: Opaque cursor for pagination. Use the cursor value from result_info of the
              previous response.

          field_changed: Filter by the field that was changed.

          order: Sort order for results. Use 'asc' for oldest first or 'desc' for newest first.

          per_page: Number of results per page.

          since: Filter entries changed at or after this timestamp (RFC 3339).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/security-center/insights/audit-log",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=AsyncCursorPagination[AuditLogListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "before": before,
                        "changed_by": changed_by,
                        "cursor": cursor,
                        "field_changed": field_changed,
                        "order": order,
                        "per_page": per_page,
                        "since": since,
                    },
                    audit_log_list_params.AuditLogListParams,
                ),
            ),
            model=AuditLogListResponse,
        )

    def list_by_insight(
        self,
        issue_id: str,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        before: Union[str, datetime] | Omit = omit,
        changed_by: str | Omit = omit,
        cursor: str | Omit = omit,
        field_changed: Literal["status", "user_classification"] | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        per_page: int | Omit = omit,
        since: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AuditLogListByInsightResponse, AsyncCursorPagination[AuditLogListByInsightResponse]]:
        """
        Lists audit log entries for a specific Security Center insight, showing changes
        to its status and classification over time.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          before: Filter entries changed before this timestamp (RFC 3339).

          changed_by: Filter by the actor that made the change.

          cursor: Opaque cursor for pagination. Use the cursor value from result_info of the
              previous response.

          field_changed: Filter by the field that was changed.

          order: Sort order for results. Use 'asc' for oldest first or 'desc' for newest first.

          per_page: Number of results per page.

          since: Filter entries changed at or after this timestamp (RFC 3339).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not issue_id:
            raise ValueError(f"Expected a non-empty value for `issue_id` but received {issue_id!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/security-center/insights/{issue_id}/audit-log",
                issue_id=issue_id,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=AsyncCursorPagination[AuditLogListByInsightResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "before": before,
                        "changed_by": changed_by,
                        "cursor": cursor,
                        "field_changed": field_changed,
                        "order": order,
                        "per_page": per_page,
                        "since": since,
                    },
                    audit_log_list_by_insight_params.AuditLogListByInsightParams,
                ),
            ),
            model=AuditLogListByInsightResponse,
        )


class AuditLogsResourceWithRawResponse:
    def __init__(self, audit_logs: AuditLogsResource) -> None:
        self._audit_logs = audit_logs

        self.list = to_raw_response_wrapper(
            audit_logs.list,
        )
        self.list_by_insight = to_raw_response_wrapper(
            audit_logs.list_by_insight,
        )


class AsyncAuditLogsResourceWithRawResponse:
    def __init__(self, audit_logs: AsyncAuditLogsResource) -> None:
        self._audit_logs = audit_logs

        self.list = async_to_raw_response_wrapper(
            audit_logs.list,
        )
        self.list_by_insight = async_to_raw_response_wrapper(
            audit_logs.list_by_insight,
        )


class AuditLogsResourceWithStreamingResponse:
    def __init__(self, audit_logs: AuditLogsResource) -> None:
        self._audit_logs = audit_logs

        self.list = to_streamed_response_wrapper(
            audit_logs.list,
        )
        self.list_by_insight = to_streamed_response_wrapper(
            audit_logs.list_by_insight,
        )


class AsyncAuditLogsResourceWithStreamingResponse:
    def __init__(self, audit_logs: AsyncAuditLogsResource) -> None:
        self._audit_logs = audit_logs

        self.list = async_to_streamed_response_wrapper(
            audit_logs.list,
        )
        self.list_by_insight = async_to_streamed_response_wrapper(
            audit_logs.list_by_insight,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .key_list_params import KeyListParams as KeyListParams
from .key_list_response import KeyListResponse as KeyListResponse
from .value_list_params import ValueListParams as ValueListParams
from .value_list_response import ValueListResponse as ValueListResponse
from .zone_tag_get_params import ZoneTagGetParams as ZoneTagGetParams
from .zone_tag_get_response import ZoneTagGetResponse as ZoneTagGetResponse
from .account_tag_get_params import AccountTagGetParams as AccountTagGetParams
from .zone_tag_update_params import ZoneTagUpdateParams as ZoneTagUpdateParams
from .account_tag_get_response import AccountTagGetResponse as AccountTagGetResponse
from .zone_tag_update_response import ZoneTagUpdateResponse as ZoneTagUpdateResponse
from .account_tag_update_params import AccountTagUpdateParams as AccountTagUpdateParams
from .account_tag_update_response import AccountTagUpdateResponse as AccountTagUpdateResponse
from .resource_tagging_list_params import ResourceTaggingListParams as ResourceTaggingListParams
from .resource_tagging_list_response import ResourceTaggingListResponse as ResourceTaggingListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ......_models import BaseModel

__all__ = ["EntryCreateResponse"]


class EntryCreateResponse(BaseModel):
    entry_id: str

    header_name: str

    num_cells: int

    upload_status: Literal["empty", "uploading", "pending", "processing", "failed", "complete"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["AuditLogListParams"]


class AuditLogListParams(TypedDict, total=False):
    account_id: str
    """The Account ID to use for this endpoint. Mutually exclusive with the Zone ID."""

    zone_id: str
    """The Zone ID to use for this endpoint. Mutually exclusive with the Account ID."""

    before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter entries changed before this timestamp (RFC 3339)."""

    changed_by: str
    """Filter by the actor that made the change."""

    cursor: str
    """Opaque cursor for pagination.

    Use the cursor value from result_info of the previous response.
    """

    field_changed: Literal["status", "user_classification"]
    """Filter by the field that was changed."""

    order: Literal["asc", "desc"]
    """Sort order for results. Use 'asc' for oldest first or 'desc' for newest first."""

    per_page: int
    """Number of results per page."""

    since: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter entries changed at or after this timestamp (RFC 3339)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...types.cache import smart_tiered_cache_edit_params, smart_tiered_cache_create_params
from ..._base_client import make_request_options
from ...types.cache.smart_tiered_cache_get_response import SmartTieredCacheGetResponse
from ...types.cache.smart_tiered_cache_edit_response import SmartTieredCacheEditResponse
from ...types.cache.smart_tiered_cache_create_response import SmartTieredCacheCreateResponse
from ...types.cache.smart_tiered_cache_delete_response import SmartTieredCacheDeleteResponse

__all__ = ["SmartTieredCacheResource", "AsyncSmartTieredCacheResource"]


class SmartTieredCacheResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SmartTieredCacheResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SmartTieredCacheResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SmartTieredCacheResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SmartTieredCacheResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        zone_id: str,
        value: Literal["on", "off"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SmartTieredCacheCreateResponse]:
        """
        Smart Tiered Cache dynamically selects the single closest upper tier for each of
        your website's origins with no configuration required, using our in-house
        performance and routing data. Cloudflare collects latency data for each request
        to an origin, and uses the latency data to determine how well any upper-tier
        data center is connected with an origin. As a result, Cloudflare can select the
        data center with the lowest latency to be the upper-tier for an origin.

        Args:
          zone_id: Identifier.

          value: Enable or disable the Smart Tiered Cache.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/cache/tiered_cache_smart_topology_enable", zone_id=zone_id),
            body=maybe_transform({"value": value}, smart_tiered_cache_create_params.SmartTieredCacheCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SmartTieredCacheCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SmartTieredCacheCreateResponse]], ResultWrapper[SmartTieredCacheCreateResponse]),
        )

    def delete(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SmartTieredCacheDeleteResponse]:
        """
        Smart Tiered Cache dynamically selects the single closest upper tier for each of
        your website’s origins with no configuration required, using our in-house
        performance and routing data. Cloudflare collects latency data for each request
        to an origin, and uses the latency data to determine how well any upper-tier
        data center is connected with an origin. As a result, Cloudflare can select the
        data center with the lowest latency to be the upper-tier for an origin.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._delete(
            path_template("/zones/{zone_id}/cache/tiered_cache_smart_topology_enable", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SmartTieredCacheDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SmartTieredCacheDeleteResponse]], ResultWrapper[SmartTieredCacheDeleteResponse]),
        )

    def edit(
        self,
        *,
        zone_id: str,
        value: Literal["on", "off"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SmartTieredCacheEditResponse]:
        """
        Smart Tiered Cache dynamically selects the single closest upper tier for each of
        your website’s origins with no configuration required, using our in-house
        performance and routing data. Cloudflare collects latency data for each request
        to an origin, and uses the latency data to determine how well any upper-tier
        data center is connected with an origin. As a result, Cloudflare can select the
        data center with the lowest latency to be the upper-tier for an origin.

        Args:
          zone_id: Identifier.

          value: Enable or disable the Smart Tiered Cache.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._patch(
            path_template("/zones/{zone_id}/cache/tiered_cache_smart_topology_enable", zone_id=zone_id),
            body=maybe_transform({"value": value}, smart_tiered_cache_edit_params.SmartTieredCacheEditParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SmartTieredCacheEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SmartTieredCacheEditResponse]], ResultWrapper[SmartTieredCacheEditResponse]),
        )

    def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SmartTieredCacheGetResponse]:
        """
        Smart Tiered Cache dynamically selects the single closest upper tier for each of
        your website’s origins with no configuration required, using our in-house
        performance and routing data. Cloudflare collects latency data for each request
        to an origin, and uses the latency data to determine how well any upper-tier
        data center is connected with an origin. As a result, Cloudflare can select the
        data center with the lowest latency to be the upper-tier for an origin.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/cache/tiered_cache_smart_topology_enable", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SmartTieredCacheGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SmartTieredCacheGetResponse]], ResultWrapper[SmartTieredCacheGetResponse]),
        )


class AsyncSmartTieredCacheResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSmartTieredCacheResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSmartTieredCacheResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSmartTieredCacheResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSmartTieredCacheResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        zone_id: str,
        value: Literal["on", "off"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SmartTieredCacheCreateResponse]:
        """
        Smart Tiered Cache dynamically selects the single closest upper tier for each of
        your website's origins with no configuration required, using our in-house
        performance and routing data. Cloudflare collects latency data for each request
        to an origin, and uses the latency data to determine how well any upper-tier
        data center is connected with an origin. As a result, Cloudflare can select the
        data center with the lowest latency to be the upper-tier for an origin.

        Args:
          zone_id: Identifier.

          value: Enable or disable the Smart Tiered Cache.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/cache/tiered_cache_smart_topology_enable", zone_id=zone_id),
            body=await async_maybe_transform(
                {"value": value}, smart_tiered_cache_create_params.SmartTieredCacheCreateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SmartTieredCacheCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SmartTieredCacheCreateResponse]], ResultWrapper[SmartTieredCacheCreateResponse]),
        )

    async def delete(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SmartTieredCacheDeleteResponse]:
        """
        Smart Tiered Cache dynamically selects the single closest upper tier for each of
        your website’s origins with no configuration required, using our in-house
        performance and routing data. Cloudflare collects latency data for each request
        to an origin, and uses the latency data to determine how well any upper-tier
        data center is connected with an origin. As a result, Cloudflare can select the
        data center with the lowest latency to be the upper-tier for an origin.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._delete(
            path_template("/zones/{zone_id}/cache/tiered_cache_smart_topology_enable", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SmartTieredCacheDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SmartTieredCacheDeleteResponse]], ResultWrapper[SmartTieredCacheDeleteResponse]),
        )

    async def edit(
        self,
        *,
        zone_id: str,
        value: Literal["on", "off"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SmartTieredCacheEditResponse]:
        """
        Smart Tiered Cache dynamically selects the single closest upper tier for each of
        your website’s origins with no configuration required, using our in-house
        performance and routing data. Cloudflare collects latency data for each request
        to an origin, and uses the latency data to determine how well any upper-tier
        data center is connected with an origin. As a result, Cloudflare can select the
        data center with the lowest latency to be the upper-tier for an origin.

        Args:
          zone_id: Identifier.

          value: Enable or disable the Smart Tiered Cache.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._patch(
            path_template("/zones/{zone_id}/cache/tiered_cache_smart_topology_enable", zone_id=zone_id),
            body=await async_maybe_transform(
                {"value": value}, smart_tiered_cache_edit_params.SmartTieredCacheEditParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SmartTieredCacheEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SmartTieredCacheEditResponse]], ResultWrapper[SmartTieredCacheEditResponse]),
        )

    async def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SmartTieredCacheGetResponse]:
        """
        Smart Tiered Cache dynamically selects the single closest upper tier for each of
        your website’s origins with no configuration required, using our in-house
        performance and routing data. Cloudflare collects latency data for each request
        to an origin, and uses the latency data to determine how well any upper-tier
        data center is connected with an origin. As a result, Cloudflare can select the
        data center with the lowest latency to be the upper-tier for an origin.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/cache/tiered_cache_smart_topology_enable", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SmartTieredCacheGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SmartTieredCacheGetResponse]], ResultWrapper[SmartTieredCacheGetResponse]),
        )


class SmartTieredCacheResourceWithRawResponse:
    def __init__(self, smart_tiered_cache: SmartTieredCacheResource) -> None:
        self._smart_tiered_cache = smart_tiered_cache

        self.create = to_raw_response_wrapper(
            smart_tiered_cache.create,
        )
        self.delete = to_raw_response_wrapper(
            smart_tiered_cache.delete,
        )
        self.edit = to_raw_response_wrapper(
            smart_tiered_cache.edit,
        )
        self.get = to_raw_response_wrapper(
            smart_tiered_cache.get,
        )


class AsyncSmartTieredCacheResourceWithRawResponse:
    def __init__(self, smart_tiered_cache: AsyncSmartTieredCacheResource) -> None:
        self._smart_tiered_cache = smart_tiered_cache

        self.create = async_to_raw_response_wrapper(
            smart_tiered_cache.create,
        )
        self.delete = async_to_raw_response_wrapper(
            smart_tiered_cache.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            smart_tiered_cache.edit,
        )
        self.get = async_to_raw_response_wrapper(
            smart_tiered_cache.get,
        )


class SmartTieredCacheResourceWithStreamingResponse:
    def __init__(self, smart_tiered_cache: SmartTieredCacheResource) -> None:
        self._smart_tiered_cache = smart_tiered_cache

        self.create = to_streamed_response_wrapper(
            smart_tiered_cache.create,
        )
        self.delete = to_streamed_response_wrapper(
            smart_tiered_cache.delete,
        )
        self.edit = to_streamed_response_wrapper(
            smart_tiered_cache.edit,
        )
        self.get = to_streamed_response_wrapper(
            smart_tiered_cache.get,
        )


class AsyncSmartTieredCacheResourceWithStreamingResponse:
    def __init__(self, smart_tiered_cache: AsyncSmartTieredCacheResource) -> None:
        self._smart_tiered_cache = smart_tiered_cache

        self.create = async_to_streamed_response_wrapper(
            smart_tiered_cache.create,
        )
        self.delete = async_to_streamed_response_wrapper(
            smart_tiered_cache.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            smart_tiered_cache.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            smart_tiered_cache.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["LocationGetResponse", "TrafficAnomaly"]


class TrafficAnomaly(BaseModel):
    client_country_alpha2: str = FieldInfo(alias="clientCountryAlpha2")

    client_country_name: str = FieldInfo(alias="clientCountryName")

    value: str
    """A numeric string."""


class LocationGetResponse(BaseModel):
    traffic_anomalies: List[TrafficAnomaly] = FieldInfo(alias="trafficAnomalies")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Required, TypedDict

__all__ = ["InstanceBulkParams", "Body", "BodyInstanceRetention"]


class InstanceBulkParams(TypedDict, total=False):
    account_id: Required[str]

    body: Iterable[Body]


class BodyInstanceRetention(TypedDict, total=False):
    error_retention: Union[int, str]
    """Specifies the duration in milliseconds or as a string like '5 minutes'."""

    success_retention: Union[int, str]
    """Specifies the duration in milliseconds or as a string like '5 minutes'."""


class Body(TypedDict, total=False):
    instance_id: str

    instance_retention: BodyInstanceRetention

    params: object



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.intel.indicator_feeds import snapshot_update_params
from ....types.intel.indicator_feeds.snapshot_update_response import SnapshotUpdateResponse

__all__ = ["SnapshotsResource", "AsyncSnapshotsResource"]


class SnapshotsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SnapshotsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SnapshotsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SnapshotsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SnapshotsResourceWithStreamingResponse(self)

    def update(
        self,
        feed_id: int,
        *,
        account_id: str,
        source: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SnapshotUpdateResponse]:
        """
        Revises the raw data entries in a custom threat indicator feed.

        Args:
          account_id: Identifier

          feed_id: Indicator feed ID

          source: The file to upload

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._put(
            path_template(
                "/accounts/{account_id}/intel/indicator-feeds/{feed_id}/snapshot",
                account_id=account_id,
                feed_id=feed_id,
            ),
            body=maybe_transform({"source": source}, snapshot_update_params.SnapshotUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SnapshotUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SnapshotUpdateResponse]], ResultWrapper[SnapshotUpdateResponse]),
        )


class AsyncSnapshotsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSnapshotsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSnapshotsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSnapshotsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSnapshotsResourceWithStreamingResponse(self)

    async def update(
        self,
        feed_id: int,
        *,
        account_id: str,
        source: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SnapshotUpdateResponse]:
        """
        Revises the raw data entries in a custom threat indicator feed.

        Args:
          account_id: Identifier

          feed_id: Indicator feed ID

          source: The file to upload

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._put(
            path_template(
                "/accounts/{account_id}/intel/indicator-feeds/{feed_id}/snapshot",
                account_id=account_id,
                feed_id=feed_id,
            ),
            body=await async_maybe_transform({"source": source}, snapshot_update_params.SnapshotUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SnapshotUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SnapshotUpdateResponse]], ResultWrapper[SnapshotUpdateResponse]),
        )


class SnapshotsResourceWithRawResponse:
    def __init__(self, snapshots: SnapshotsResource) -> None:
        self._snapshots = snapshots

        self.update = to_raw_response_wrapper(
            snapshots.update,
        )


class AsyncSnapshotsResourceWithRawResponse:
    def __init__(self, snapshots: AsyncSnapshotsResource) -> None:
        self._snapshots = snapshots

        self.update = async_to_raw_response_wrapper(
            snapshots.update,
        )


class SnapshotsResourceWithStreamingResponse:
    def __init__(self, snapshots: SnapshotsResource) -> None:
        self._snapshots = snapshots

        self.update = to_streamed_response_wrapper(
            snapshots.update,
        )


class AsyncSnapshotsResourceWithStreamingResponse:
    def __init__(self, snapshots: AsyncSnapshotsResource) -> None:
        self._snapshots = snapshots

        self.update = async_to_streamed_response_wrapper(
            snapshots.update,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["SpendingLimitGetResponse", "Config"]


class Config(BaseModel):
    amount: Optional[float] = None

    duration: Optional[str] = None

    strategy: Optional[str] = None


class SpendingLimitGetResponse(BaseModel):
    config: Config

    enabled: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["ApplicationGetResponse"]


class ApplicationGetResponse(BaseModel):
    id: str
    """Returns the application ID."""

    application_confidence_score: float
    """Confidence score for the application. Returns -1 when no score is available."""

    application_source: str
    """Returns the application source."""

    application_type: str
    """Returns the application type."""

    application_type_description: str
    """Returns the application type description."""

    created_at: str
    """Returns the application creation time."""

    gen_ai_score: float
    """GenAI score for the application. Returns -1 when no score is available."""

    hostnames: List[str]
    """Returns the list of hostnames for the application."""

    human_id: str
    """Returns the human readable ID."""

    ip_subnets: List[str]
    """Returns the list of IP subnets for the application."""

    name: str
    """Returns the application name."""

    port_protocols: List[str]
    """Returns the list of port protocols for the application."""

    support_domains: List[str]
    """Returns the list of support domains for the application."""

    supported: List[Literal["GATEWAY", "ACCESS", "CASB"]]
    """Cloudflare products that support this application."""

    updated_at: str
    """Returns the application update time."""

    version: str
    """Returns the application version."""

    application_score_composition: Optional[object] = None
    """Returns the score composition breakdown for the application."""

    intel_id: Optional[int] = None
    """Returns the Intel API ID for the application."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .fallthrough_create_params import FallthroughCreateParams as FallthroughCreateParams
from .fallthrough_create_response import FallthroughCreateResponse as FallthroughCreateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["AdditionalRoutesParam"]


class AdditionalRoutesParam(TypedDict, total=False):
    host: str
    """The hostname to which this waiting room will be applied (no wildcards).

    The hostname must be the primary domain, subdomain, or custom hostname (if using
    SSL for SaaS) of this zone. Please do not include the scheme (http:// or
    https://).
    """

    path: str
    """Sets the path within the host to enable the waiting room on.

    The waiting room will be enabled for all subpaths as well. If there are two
    waiting rooms on the same subpath, the waiting room for the most specific path
    will be chosen. Wildcards and query parameters are not supported.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ...._base_client import AsyncPaginator, make_request_options
from ....types.email_security.settings import (
    trusted_domain_edit_params,
    trusted_domain_list_params,
    trusted_domain_create_params,
)
from ....types.email_security.settings.trusted_domain_get_response import TrustedDomainGetResponse
from ....types.email_security.settings.trusted_domain_edit_response import TrustedDomainEditResponse
from ....types.email_security.settings.trusted_domain_list_response import TrustedDomainListResponse
from ....types.email_security.settings.trusted_domain_create_response import TrustedDomainCreateResponse
from ....types.email_security.settings.trusted_domain_delete_response import TrustedDomainDeleteResponse

__all__ = ["TrustedDomainsResource", "AsyncTrustedDomainsResource"]


class TrustedDomainsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TrustedDomainsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TrustedDomainsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TrustedDomainsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TrustedDomainsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        is_recent: bool,
        is_regex: bool,
        is_similarity: bool,
        pattern: str,
        comments: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TrustedDomainCreateResponse]:
        """Creates a new trusted domain pattern.

        Use for partner domains or approved
        senders that should bypass recent domain registration and similarity checks.
        Configure whether it prevents recent domain or spoof dispositions.

        Args:
          account_id: Identifier.

          is_recent: Select to prevent recently registered domains from triggering a Suspicious or
              Malicious disposition.

          is_similarity: Select for partner or other approved domains that have similar spelling to your
              connected domains. Prevents listed domains from triggering a Spoof disposition.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/email-security/settings/trusted_domains", account_id=account_id),
            body=maybe_transform(
                {
                    "is_recent": is_recent,
                    "is_regex": is_regex,
                    "is_similarity": is_similarity,
                    "pattern": pattern,
                    "comments": comments,
                },
                trusted_domain_create_params.TrustedDomainCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TrustedDomainCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TrustedDomainCreateResponse]], ResultWrapper[TrustedDomainCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        is_recent: bool | Omit = omit,
        is_similarity: bool | Omit = omit,
        order: Literal["pattern", "created_at"] | Omit = omit,
        page: int | Omit = omit,
        pattern: str | Omit = omit,
        per_page: int | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[TrustedDomainListResponse]:
        """Returns a paginated list of trusted domain patterns.

        Trusted domains prevent
        false positives for recently registered domains and lookalike domain detections.
        Patterns can use regular expressions for flexible matching.

        Args:
          account_id: Identifier.

          direction: The sorting direction.

          is_recent: Filter to show only recently registered domains that are trusted to prevent
              triggering Suspicious or Malicious dispositions.

          is_similarity: Filter to show only proximity domains (partner or approved domains with similar
              spelling to connected domains) that prevent Spoof dispositions.

          order: Field to sort by.

          page: Current page within paginated list of results.

          per_page: The number of results per page. Maximum value is 1000.

          search: Search term for filtering records. Behavior may change.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/email-security/settings/trusted_domains", account_id=account_id),
            page=SyncV4PagePaginationArray[TrustedDomainListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "is_recent": is_recent,
                        "is_similarity": is_similarity,
                        "order": order,
                        "page": page,
                        "pattern": pattern,
                        "per_page": per_page,
                        "search": search,
                    },
                    trusted_domain_list_params.TrustedDomainListParams,
                ),
            ),
            model=TrustedDomainListResponse,
        )

    def delete(
        self,
        trusted_domain_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TrustedDomainDeleteResponse]:
        """Removes a trusted domain pattern.

        After deletion, emails from this domain will
        be subject to normal recent domain and similarity checks.

        Args:
          account_id: Identifier.

          trusted_domain_id: Trusted domain identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not trusted_domain_id:
            raise ValueError(f"Expected a non-empty value for `trusted_domain_id` but received {trusted_domain_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/email-security/settings/trusted_domains/{trusted_domain_id}",
                account_id=account_id,
                trusted_domain_id=trusted_domain_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TrustedDomainDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TrustedDomainDeleteResponse]], ResultWrapper[TrustedDomainDeleteResponse]),
        )

    def edit(
        self,
        trusted_domain_id: str,
        *,
        account_id: str,
        comments: Optional[str] | Omit = omit,
        is_recent: bool | Omit = omit,
        is_regex: bool | Omit = omit,
        is_similarity: bool | Omit = omit,
        pattern: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TrustedDomainEditResponse]:
        """Updates an existing trusted domain pattern.

        Only provided fields will be
        modified. Changes take effect for new emails matching the pattern.

        Args:
          account_id: Identifier.

          trusted_domain_id: Trusted domain identifier

          is_recent: Select to prevent recently registered domains from triggering a Suspicious or
              Malicious disposition.

          is_similarity: Select for partner or other approved domains that have similar spelling to your
              connected domains. Prevents listed domains from triggering a Spoof disposition.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not trusted_domain_id:
            raise ValueError(f"Expected a non-empty value for `trusted_domain_id` but received {trusted_domain_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/email-security/settings/trusted_domains/{trusted_domain_id}",
                account_id=account_id,
                trusted_domain_id=trusted_domain_id,
            ),
            body=maybe_transform(
                {
                    "comments": comments,
                    "is_recent": is_recent,
                    "is_regex": is_regex,
                    "is_similarity": is_similarity,
                    "pattern": pattern,
                },
                trusted_domain_edit_params.TrustedDomainEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TrustedDomainEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TrustedDomainEditResponse]], ResultWrapper[TrustedDomainEditResponse]),
        )

    def get(
        self,
        trusted_domain_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TrustedDomainGetResponse]:
        """
        Retrieves details for a specific trusted domain pattern including its pattern
        value, whether it uses regex matching, and which detection types it affects.

        Args:
          account_id: Identifier.

          trusted_domain_id: Trusted domain identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not trusted_domain_id:
            raise ValueError(f"Expected a non-empty value for `trusted_domain_id` but received {trusted_domain_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/email-security/settings/trusted_domains/{trusted_domain_id}",
                account_id=account_id,
                trusted_domain_id=trusted_domain_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TrustedDomainGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TrustedDomainGetResponse]], ResultWrapper[TrustedDomainGetResponse]),
        )


class AsyncTrustedDomainsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTrustedDomainsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTrustedDomainsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTrustedDomainsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTrustedDomainsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        is_recent: bool,
        is_regex: bool,
        is_similarity: bool,
        pattern: str,
        comments: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TrustedDomainCreateResponse]:
        """Creates a new trusted domain pattern.

        Use for partner domains or approved
        senders that should bypass recent domain registration and similarity checks.
        Configure whether it prevents recent domain or spoof dispositions.

        Args:
          account_id: Identifier.

          is_recent: Select to prevent recently registered domains from triggering a Suspicious or
              Malicious disposition.

          is_similarity: Select for partner or other approved domains that have similar spelling to your
              connected domains. Prevents listed domains from triggering a Spoof disposition.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/email-security/settings/trusted_domains", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "is_recent": is_recent,
                    "is_regex": is_regex,
                    "is_similarity": is_similarity,
                    "pattern": pattern,
                    "comments": comments,
                },
                trusted_domain_create_params.TrustedDomainCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TrustedDomainCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TrustedDomainCreateResponse]], ResultWrapper[TrustedDomainCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        is_recent: bool | Omit = omit,
        is_similarity: bool | Omit = omit,
        order: Literal["pattern", "created_at"] | Omit = omit,
        page: int | Omit = omit,
        pattern: str | Omit = omit,
        per_page: int | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[TrustedDomainListResponse, AsyncV4PagePaginationArray[TrustedDomainListResponse]]:
        """Returns a paginated list of trusted domain patterns.

        Trusted domains prevent
        false positives for recently registered domains and lookalike domain detections.
        Patterns can use regular expressions for flexible matching.

        Args:
          account_id: Identifier.

          direction: The sorting direction.

          is_recent: Filter to show only recently registered domains that are trusted to prevent
              triggering Suspicious or Malicious dispositions.

          is_similarity: Filter to show only proximity domains (partner or approved domains with similar
              spelling to connected domains) that prevent Spoof dispositions.

          order: Field to sort by.

          page: Current page within paginated list of results.

          per_page: The number of results per page. Maximum value is 1000.

          search: Search term for filtering records. Behavior may change.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/email-security/settings/trusted_domains", account_id=account_id),
            page=AsyncV4PagePaginationArray[TrustedDomainListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "is_recent": is_recent,
                        "is_similarity": is_similarity,
                        "order": order,
                        "page": page,
                        "pattern": pattern,
                        "per_page": per_page,
                        "search": search,
                    },
                    trusted_domain_list_params.TrustedDomainListParams,
                ),
            ),
            model=TrustedDomainListResponse,
        )

    async def delete(
        self,
        trusted_domain_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TrustedDomainDeleteResponse]:
        """Removes a trusted domain pattern.

        After deletion, emails from this domain will
        be subject to normal recent domain and similarity checks.

        Args:
          account_id: Identifier.

          trusted_domain_id: Trusted domain identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not trusted_domain_id:
            raise ValueError(f"Expected a non-empty value for `trusted_domain_id` but received {trusted_domain_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/email-security/settings/trusted_domains/{trusted_domain_id}",
                account_id=account_id,
                trusted_domain_id=trusted_domain_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TrustedDomainDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TrustedDomainDeleteResponse]], ResultWrapper[TrustedDomainDeleteResponse]),
        )

    async def edit(
        self,
        trusted_domain_id: str,
        *,
        account_id: str,
        comments: Optional[str] | Omit = omit,
        is_recent: bool | Omit = omit,
        is_regex: bool | Omit = omit,
        is_similarity: bool | Omit = omit,
        pattern: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TrustedDomainEditResponse]:
        """Updates an existing trusted domain pattern.

        Only provided fields will be
        modified. Changes take effect for new emails matching the pattern.

        Args:
          account_id: Identifier.

          trusted_domain_id: Trusted domain identifier

          is_recent: Select to prevent recently registered domains from triggering a Suspicious or
              Malicious disposition.

          is_similarity: Select for partner or other approved domains that have similar spelling to your
              connected domains. Prevents listed domains from triggering a Spoof disposition.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not trusted_domain_id:
            raise ValueError(f"Expected a non-empty value for `trusted_domain_id` but received {trusted_domain_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/email-security/settings/trusted_domains/{trusted_domain_id}",
                account_id=account_id,
                trusted_domain_id=trusted_domain_id,
            ),
            body=await async_maybe_transform(
                {
                    "comments": comments,
                    "is_recent": is_recent,
                    "is_regex": is_regex,
                    "is_similarity": is_similarity,
                    "pattern": pattern,
                },
                trusted_domain_edit_params.TrustedDomainEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TrustedDomainEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TrustedDomainEditResponse]], ResultWrapper[TrustedDomainEditResponse]),
        )

    async def get(
        self,
        trusted_domain_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TrustedDomainGetResponse]:
        """
        Retrieves details for a specific trusted domain pattern including its pattern
        value, whether it uses regex matching, and which detection types it affects.

        Args:
          account_id: Identifier.

          trusted_domain_id: Trusted domain identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not trusted_domain_id:
            raise ValueError(f"Expected a non-empty value for `trusted_domain_id` but received {trusted_domain_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/email-security/settings/trusted_domains/{trusted_domain_id}",
                account_id=account_id,
                trusted_domain_id=trusted_domain_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TrustedDomainGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[TrustedDomainGetResponse]], ResultWrapper[TrustedDomainGetResponse]),
        )


class TrustedDomainsResourceWithRawResponse:
    def __init__(self, trusted_domains: TrustedDomainsResource) -> None:
        self._trusted_domains = trusted_domains

        self.create = to_raw_response_wrapper(
            trusted_domains.create,
        )
        self.list = to_raw_response_wrapper(
            trusted_domains.list,
        )
        self.delete = to_raw_response_wrapper(
            trusted_domains.delete,
        )
        self.edit = to_raw_response_wrapper(
            trusted_domains.edit,
        )
        self.get = to_raw_response_wrapper(
            trusted_domains.get,
        )


class AsyncTrustedDomainsResourceWithRawResponse:
    def __init__(self, trusted_domains: AsyncTrustedDomainsResource) -> None:
        self._trusted_domains = trusted_domains

        self.create = async_to_raw_response_wrapper(
            trusted_domains.create,
        )
        self.list = async_to_raw_response_wrapper(
            trusted_domains.list,
        )
        self.delete = async_to_raw_response_wrapper(
            trusted_domains.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            trusted_domains.edit,
        )
        self.get = async_to_raw_response_wrapper(
            trusted_domains.get,
        )


class TrustedDomainsResourceWithStreamingResponse:
    def __init__(self, trusted_domains: TrustedDomainsResource) -> None:
        self._trusted_domains = trusted_domains

        self.create = to_streamed_response_wrapper(
            trusted_domains.create,
        )
        self.list = to_streamed_response_wrapper(
            trusted_domains.list,
        )
        self.delete = to_streamed_response_wrapper(
            trusted_domains.delete,
        )
        self.edit = to_streamed_response_wrapper(
            trusted_domains.edit,
        )
        self.get = to_streamed_response_wrapper(
            trusted_domains.get,
        )


class AsyncTrustedDomainsResourceWithStreamingResponse:
    def __init__(self, trusted_domains: AsyncTrustedDomainsResource) -> None:
        self._trusted_domains = trusted_domains

        self.create = async_to_streamed_response_wrapper(
            trusted_domains.create,
        )
        self.list = async_to_streamed_response_wrapper(
            trusted_domains.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            trusted_domains.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            trusted_domains.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            trusted_domains.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["DCVMethod"]

DCVMethod: TypeAlias = Literal["http", "txt", "email"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from .domains import (
    DomainsResource,
    AsyncDomainsResource,
    DomainsResourceWithRawResponse,
    AsyncDomainsResourceWithRawResponse,
    DomainsResourceWithStreamingResponse,
    AsyncDomainsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from .registrations import (
    RegistrationsResource,
    AsyncRegistrationsResource,
    RegistrationsResourceWithRawResponse,
    AsyncRegistrationsResourceWithRawResponse,
    RegistrationsResourceWithStreamingResponse,
    AsyncRegistrationsResourceWithStreamingResponse,
)
from .update_status import (
    UpdateStatusResource,
    AsyncUpdateStatusResource,
    UpdateStatusResourceWithRawResponse,
    AsyncUpdateStatusResourceWithRawResponse,
    UpdateStatusResourceWithStreamingResponse,
    AsyncUpdateStatusResourceWithStreamingResponse,
)
from ..._base_client import make_request_options
from ...types.registrar import registrar_check_params, registrar_search_params
from .registration_status import (
    RegistrationStatusResource,
    AsyncRegistrationStatusResource,
    RegistrationStatusResourceWithRawResponse,
    AsyncRegistrationStatusResourceWithRawResponse,
    RegistrationStatusResourceWithStreamingResponse,
    AsyncRegistrationStatusResourceWithStreamingResponse,
)
from ...types.registrar.registrar_check_response import RegistrarCheckResponse
from ...types.registrar.registrar_search_response import RegistrarSearchResponse

__all__ = ["RegistrarResource", "AsyncRegistrarResource"]


class RegistrarResource(SyncAPIResource):
    @cached_property
    def domains(self) -> DomainsResource:
        return DomainsResource(self._client)

    @cached_property
    def registrations(self) -> RegistrationsResource:
        return RegistrationsResource(self._client)

    @cached_property
    def registration_status(self) -> RegistrationStatusResource:
        return RegistrationStatusResource(self._client)

    @cached_property
    def update_status(self) -> UpdateStatusResource:
        return UpdateStatusResource(self._client)

    @cached_property
    def with_raw_response(self) -> RegistrarResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RegistrarResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RegistrarResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RegistrarResourceWithStreamingResponse(self)

    def check(
        self,
        *,
        account_id: str,
        domains: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RegistrarCheckResponse:
        """
        Performs real-time, authoritative availability checks directly against domain
        registries. Use this endpoint to verify a domain is available before attempting
        registration via `POST /registrations`.

        **Important:** Unlike the Search endpoint, these results are authoritative and
        reflect current registry status. Always check availability immediately before
        registration as domain status can change rapidly.

        **Note:** This endpoint uses POST to accept a list of domains in the request
        body. It is a read-only operation — it does not create, modify, or reserve any
        domains.

        ### Extension support

        Only domains on extensions supported for programmatic registration by this API
        can be registered. If you check a domain on an unsupported extension, the
        response will include `registrable: false` with a `reason` field explaining why:

        - `extension_not_supported_via_api` — Cloudflare Registrar supports this
          extension in the dashboard, but it is not yet available for programmatic
          registration via this API. Register via
          `https://dash.cloudflare.com/{account_id}/domains/registrations` instead.
        - `extension_not_supported` — This extension is not supported by Cloudflare
          Registrar.
        - `extension_disallows_registration` — The extension's registry has temporarily
          or permanently frozen new registrations. No registrar can register domains on
          this extension at this time.
        - `domain_premium` — The domain is premium priced. Premium registration is not
          currently supported by this API.
        - `domain_unavailable` — The domain is already registered, reserved, or
          otherwise not available for registration on a supported extension.

        The `reason` field is only present when `registrable` is `false`.

        ### Behavior

        - Maximum 20 domains per request
        - Pricing is only returned for domains where `registrable: true`
        - Results are not cached; each request queries the registry

        ### Workflow

        1. Call this endpoint with domains the user wants to register.
        2. For each domain where `registrable: true`, present pricing to the user.
        3. If `tier: premium`, note that premium registration is not currently supported
           by this API and do not proceed to `POST /registrations`.
        4. Proceed to `POST /registrations` only for supported non-premium domains.

        Args:
          account_id: Identifier

          domains: List of fully qualified domain names (FQDNs) to check for availability. Each
              domain must include the extension.

              - Minimum: 1 domain
              - Maximum: 20 domains per request
              - Domains on unsupported extensions are returned with `registrable: false` and a
                `reason` field
              - Malformed domain names (e.g., missing extension) may be omitted from the
                response

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/registrar/domain-check", account_id=account_id),
            body=maybe_transform({"domains": domains}, registrar_check_params.RegistrarCheckParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[RegistrarCheckResponse]._unwrapper,
            ),
            cast_to=cast(Type[RegistrarCheckResponse], ResultWrapper[RegistrarCheckResponse]),
        )

    def search(
        self,
        *,
        account_id: str,
        q: str,
        extensions: SequenceNotStr[str] | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RegistrarSearchResponse:
        """
        Searches for domain name suggestions based on a keyword, phrase, or partial
        domain name. Returns a list of potentially available domains with pricing
        information.

        **Important:** Results are non-authoritative and based on cached data. Always
        use the `/domain-check` endpoint to verify real-time availability before
        attempting registration.

        Suggestions are scoped to extensions supported for programmatic registration via
        this API (`POST /registrations`). Domains on unsupported extensions will not
        appear in results, even if they are available at the registry level.

        ### Use cases

        - Brand name discovery (e.g., "acme corp" → acmecorp.com, acmecorp.dev)
        - Keyword-based suggestions (e.g., "coffee shop" → coffeeshop.com,
          mycoffeeshop.net)
        - Alternative extension discovery (e.g., "example.com" → example.com,
          example.app, example.xyz)

        ### Workflow

        1. Call this endpoint with a keyword or domain name.
        2. Present suggestions to the user.
        3. Call `/domain-check` with the user's chosen domains to confirm real-time
           availability and pricing.
        4. Proceed to `POST /registrations` only for supported non-premium domains where
           the Check response returns `registrable: true`.

        **Note:** Searching with just a domain extension (e.g., "com" or ".app") is not
        supported. Provide a keyword or domain name.

        Args:
          account_id: Identifier

          q: The search term to find domain suggestions. Accepts keywords, phrases, or full
              domain names.

              - Phrases: "coffee shop" returns coffeeshop.com, mycoffeeshop.net, etc.
              - Domain names: "example.com" returns example.com and variations across
                extensions

          extensions: Limits results to specific domain extensions from the supported set. If not
              specified, returns results across all supported extensions. Extensions not in
              the supported set are silently ignored.

          limit: Maximum number of domain suggestions to return. Defaults to 20 if not specified.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/registrar/domain-search", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "q": q,
                        "extensions": extensions,
                        "limit": limit,
                    },
                    registrar_search_params.RegistrarSearchParams,
                ),
                post_parser=ResultWrapper[RegistrarSearchResponse]._unwrapper,
            ),
            cast_to=cast(Type[RegistrarSearchResponse], ResultWrapper[RegistrarSearchResponse]),
        )


class AsyncRegistrarResource(AsyncAPIResource):
    @cached_property
    def domains(self) -> AsyncDomainsResource:
        return AsyncDomainsResource(self._client)

    @cached_property
    def registrations(self) -> AsyncRegistrationsResource:
        return AsyncRegistrationsResource(self._client)

    @cached_property
    def registration_status(self) -> AsyncRegistrationStatusResource:
        return AsyncRegistrationStatusResource(self._client)

    @cached_property
    def update_status(self) -> AsyncUpdateStatusResource:
        return AsyncUpdateStatusResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRegistrarResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRegistrarResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRegistrarResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRegistrarResourceWithStreamingResponse(self)

    async def check(
        self,
        *,
        account_id: str,
        domains: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RegistrarCheckResponse:
        """
        Performs real-time, authoritative availability checks directly against domain
        registries. Use this endpoint to verify a domain is available before attempting
        registration via `POST /registrations`.

        **Important:** Unlike the Search endpoint, these results are authoritative and
        reflect current registry status. Always check availability immediately before
        registration as domain status can change rapidly.

        **Note:** This endpoint uses POST to accept a list of domains in the request
        body. It is a read-only operation — it does not create, modify, or reserve any
        domains.

        ### Extension support

        Only domains on extensions supported for programmatic registration by this API
        can be registered. If you check a domain on an unsupported extension, the
        response will include `registrable: false` with a `reason` field explaining why:

        - `extension_not_supported_via_api` — Cloudflare Registrar supports this
          extension in the dashboard, but it is not yet available for programmatic
          registration via this API. Register via
          `https://dash.cloudflare.com/{account_id}/domains/registrations` instead.
        - `extension_not_supported` — This extension is not supported by Cloudflare
          Registrar.
        - `extension_disallows_registration` — The extension's registry has temporarily
          or permanently frozen new registrations. No registrar can register domains on
          this extension at this time.
        - `domain_premium` — The domain is premium priced. Premium registration is not
          currently supported by this API.
        - `domain_unavailable` — The domain is already registered, reserved, or
          otherwise not available for registration on a supported extension.

        The `reason` field is only present when `registrable` is `false`.

        ### Behavior

        - Maximum 20 domains per request
        - Pricing is only returned for domains where `registrable: true`
        - Results are not cached; each request queries the registry

        ### Workflow

        1. Call this endpoint with domains the user wants to register.
        2. For each domain where `registrable: true`, present pricing to the user.
        3. If `tier: premium`, note that premium registration is not currently supported
           by this API and do not proceed to `POST /registrations`.
        4. Proceed to `POST /registrations` only for supported non-premium domains.

        Args:
          account_id: Identifier

          domains: List of fully qualified domain names (FQDNs) to check for availability. Each
              domain must include the extension.

              - Minimum: 1 domain
              - Maximum: 20 domains per request
              - Domains on unsupported extensions are returned with `registrable: false` and a
                `reason` field
              - Malformed domain names (e.g., missing extension) may be omitted from the
                response

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/registrar/domain-check", account_id=account_id),
            body=await async_maybe_transform({"domains": domains}, registrar_check_params.RegistrarCheckParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[RegistrarCheckResponse]._unwrapper,
            ),
            cast_to=cast(Type[RegistrarCheckResponse], ResultWrapper[RegistrarCheckResponse]),
        )

    async def search(
        self,
        *,
        account_id: str,
        q: str,
        extensions: SequenceNotStr[str] | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RegistrarSearchResponse:
        """
        Searches for domain name suggestions based on a keyword, phrase, or partial
        domain name. Returns a list of potentially available domains with pricing
        information.

        **Important:** Results are non-authoritative and based on cached data. Always
        use the `/domain-check` endpoint to verify real-time availability before
        attempting registration.

        Suggestions are scoped to extensions supported for programmatic registration via
        this API (`POST /registrations`). Domains on unsupported extensions will not
        appear in results, even if they are available at the registry level.

        ### Use cases

        - Brand name discovery (e.g., "acme corp" → acmecorp.com, acmecorp.dev)
        - Keyword-based suggestions (e.g., "coffee shop" → coffeeshop.com,
          mycoffeeshop.net)
        - Alternative extension discovery (e.g., "example.com" → example.com,
          example.app, example.xyz)

        ### Workflow

        1. Call this endpoint with a keyword or domain name.
        2. Present suggestions to the user.
        3. Call `/domain-check` with the user's chosen domains to confirm real-time
           availability and pricing.
        4. Proceed to `POST /registrations` only for supported non-premium domains where
           the Check response returns `registrable: true`.

        **Note:** Searching with just a domain extension (e.g., "com" or ".app") is not
        supported. Provide a keyword or domain name.

        Args:
          account_id: Identifier

          q: The search term to find domain suggestions. Accepts keywords, phrases, or full
              domain names.

              - Phrases: "coffee shop" returns coffeeshop.com, mycoffeeshop.net, etc.
              - Domain names: "example.com" returns example.com and variations across
                extensions

          extensions: Limits results to specific domain extensions from the supported set. If not
              specified, returns results across all supported extensions. Extensions not in
              the supported set are silently ignored.

          limit: Maximum number of domain suggestions to return. Defaults to 20 if not specified.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/registrar/domain-search", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "q": q,
                        "extensions": extensions,
                        "limit": limit,
                    },
                    registrar_search_params.RegistrarSearchParams,
                ),
                post_parser=ResultWrapper[RegistrarSearchResponse]._unwrapper,
            ),
            cast_to=cast(Type[RegistrarSearchResponse], ResultWrapper[RegistrarSearchResponse]),
        )


class RegistrarResourceWithRawResponse:
    def __init__(self, registrar: RegistrarResource) -> None:
        self._registrar = registrar

        self.check = to_raw_response_wrapper(
            registrar.check,
        )
        self.search = to_raw_response_wrapper(
            registrar.search,
        )

    @cached_property
    def domains(self) -> DomainsResourceWithRawResponse:
        return DomainsResourceWithRawResponse(self._registrar.domains)

    @cached_property
    def registrations(self) -> RegistrationsResourceWithRawResponse:
        return RegistrationsResourceWithRawResponse(self._registrar.registrations)

    @cached_property
    def registration_status(self) -> RegistrationStatusResourceWithRawResponse:
        return RegistrationStatusResourceWithRawResponse(self._registrar.registration_status)

    @cached_property
    def update_status(self) -> UpdateStatusResourceWithRawResponse:
        return UpdateStatusResourceWithRawResponse(self._registrar.update_status)


class AsyncRegistrarResourceWithRawResponse:
    def __init__(self, registrar: AsyncRegistrarResource) -> None:
        self._registrar = registrar

        self.check = async_to_raw_response_wrapper(
            registrar.check,
        )
        self.search = async_to_raw_response_wrapper(
            registrar.search,
        )

    @cached_property
    def domains(self) -> AsyncDomainsResourceWithRawResponse:
        return AsyncDomainsResourceWithRawResponse(self._registrar.domains)

    @cached_property
    def registrations(self) -> AsyncRegistrationsResourceWithRawResponse:
        return AsyncRegistrationsResourceWithRawResponse(self._registrar.registrations)

    @cached_property
    def registration_status(self) -> AsyncRegistrationStatusResourceWithRawResponse:
        return AsyncRegistrationStatusResourceWithRawResponse(self._registrar.registration_status)

    @cached_property
    def update_status(self) -> AsyncUpdateStatusResourceWithRawResponse:
        return AsyncUpdateStatusResourceWithRawResponse(self._registrar.update_status)


class RegistrarResourceWithStreamingResponse:
    def __init__(self, registrar: RegistrarResource) -> None:
        self._registrar = registrar

        self.check = to_streamed_response_wrapper(
            registrar.check,
        )
        self.search = to_streamed_response_wrapper(
            registrar.search,
        )

    @cached_property
    def domains(self) -> DomainsResourceWithStreamingResponse:
        return DomainsResourceWithStreamingResponse(self._registrar.domains)

    @cached_property
    def registrations(self) -> RegistrationsResourceWithStreamingResponse:
        return RegistrationsResourceWithStreamingResponse(self._registrar.registrations)

    @cached_property
    def registration_status(self) -> RegistrationStatusResourceWithStreamingResponse:
        return RegistrationStatusResourceWithStreamingResponse(self._registrar.registration_status)

    @cached_property
    def update_status(self) -> UpdateStatusResourceWithStreamingResponse:
        return UpdateStatusResourceWithStreamingResponse(self._registrar.update_status)


class AsyncRegistrarResourceWithStreamingResponse:
    def __init__(self, registrar: AsyncRegistrarResource) -> None:
        self._registrar = registrar

        self.check = async_to_streamed_response_wrapper(
            registrar.check,
        )
        self.search = async_to_streamed_response_wrapper(
            registrar.search,
        )

    @cached_property
    def domains(self) -> AsyncDomainsResourceWithStreamingResponse:
        return AsyncDomainsResourceWithStreamingResponse(self._registrar.domains)

    @cached_property
    def registrations(self) -> AsyncRegistrationsResourceWithStreamingResponse:
        return AsyncRegistrationsResourceWithStreamingResponse(self._registrar.registrations)

    @cached_property
    def registration_status(self) -> AsyncRegistrationStatusResourceWithStreamingResponse:
        return AsyncRegistrationStatusResourceWithStreamingResponse(self._registrar.registration_status)

    @cached_property
    def update_status(self) -> AsyncUpdateStatusResourceWithStreamingResponse:
        return AsyncUpdateStatusResourceWithStreamingResponse(self._registrar.update_status)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .config import (
    ConfigResource,
    AsyncConfigResource,
    ConfigResourceWithRawResponse,
    AsyncConfigResourceWithRawResponse,
    ConfigResourceWithStreamingResponse,
    AsyncConfigResourceWithStreamingResponse,
)
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["CmbResource", "AsyncCmbResource"]


class CmbResource(SyncAPIResource):
    @cached_property
    def config(self) -> ConfigResource:
        return ConfigResource(self._client)

    @cached_property
    def with_raw_response(self) -> CmbResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CmbResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CmbResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CmbResourceWithStreamingResponse(self)


class AsyncCmbResource(AsyncAPIResource):
    @cached_property
    def config(self) -> AsyncConfigResource:
        return AsyncConfigResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCmbResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCmbResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCmbResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCmbResourceWithStreamingResponse(self)


class CmbResourceWithRawResponse:
    def __init__(self, cmb: CmbResource) -> None:
        self._cmb = cmb

    @cached_property
    def config(self) -> ConfigResourceWithRawResponse:
        return ConfigResourceWithRawResponse(self._cmb.config)


class AsyncCmbResourceWithRawResponse:
    def __init__(self, cmb: AsyncCmbResource) -> None:
        self._cmb = cmb

    @cached_property
    def config(self) -> AsyncConfigResourceWithRawResponse:
        return AsyncConfigResourceWithRawResponse(self._cmb.config)


class CmbResourceWithStreamingResponse:
    def __init__(self, cmb: CmbResource) -> None:
        self._cmb = cmb

    @cached_property
    def config(self) -> ConfigResourceWithStreamingResponse:
        return ConfigResourceWithStreamingResponse(self._cmb.config)


class AsyncCmbResourceWithStreamingResponse:
    def __init__(self, cmb: AsyncCmbResource) -> None:
        self._cmb = cmb

    @cached_property
    def config(self) -> AsyncConfigResourceWithStreamingResponse:
        return AsyncConfigResourceWithStreamingResponse(self._cmb.config)
